#
# Python bindings for libparted (built on top of the _ped Python module).
#
# Copyright The pyparted Project Authors
# SPDX-License-Identifier: GPL-2.0-or-later
#

import locale
import functools


def localeC(fn):
    # setlocale is not thread-safe, and anaconda (at least) may call this from
    # another thread.  This is just a luxury to have untranslated tracebacks,
    # so it's not worth tracebacking itself.
    def _setlocale(l):
        try:
            locale.setlocale(locale.LC_MESSAGES, l)
        # pylint: disable=bare-except
        except:
            pass

    @functools.wraps(fn)
    def new(*args, **kwds):
        oldlocale = locale.getlocale(locale.LC_MESSAGES)
        _setlocale("C")
        try:
            ret = fn(*args, **kwds)
        finally:
            _setlocale(oldlocale)
        return ret

    return new



#
# partition.py
# Python bindings for libparted (built on top of the _ped Python module).
#
# Copyright The pyparted Project Authors
# SPDX-License-Identifier: GPL-2.0-or-later
#

import math
import warnings

import _ped
import parted

from parted.decorators import localeC

# XXX: add docstrings


class Partition(object):
    # pylint: disable=W0622
    @localeC
    def __init__(self, disk=None, type=None, fs=None, geometry=None, PedPartition=None):
        if PedPartition is None:
            if disk is None:
                raise parted.PartitionException("no disk specified")
            elif type is None:
                raise parted.PartitionException("no type specified")
            elif geometry is None:
                raise parted.PartitionException("no geometry specified")

            self._fileSystem = fs
            self._geometry = geometry
            self._disk = disk

            if fs is None:
                self.__partition = _ped.Partition(
                    disk.getPedDisk(), type, geometry.start, geometry.end
                )
            else:
                self.__partition = _ped.Partition(
                    disk.getPedDisk(),
                    type,
                    geometry.start,
                    geometry.end,
                    parted.fileSystemType[fs.type],
                )
        else:
            self.__partition = PedPartition
            self._geometry = parted.Geometry(PedGeometry=self.__partition.geom)

            if disk is None:
                self._disk = parted.Disk(PedDisk=self.__partition.disk)
            else:
                self._disk = disk

            if self.__partition.fs_type is None:
                self._fileSystem = None
            else:
                # pylint: disable=E1103
                self._fileSystem = parted.FileSystem(
                    type=self.__partition.fs_type.name, geometry=self._geometry
                )

    def __eq__(self, other):
        return not self.__ne__(other)

    def __ne__(self, other):
        if not isinstance(self, other.__class__):
            return True

        return (
            self.path != other.path
            or self.type != other.type
            or self.geometry != other.geometry
            or self.fileSystem != other.fileSystem
        )

    def __str__(self):
        try:
            name = self.name
        except parted.PartitionException:
            name = None

        s = (
            "parted.Partition instance --\n"
            "  disk: %(disk)r  fileSystem: %(fileSystem)r\n"
            "  number: %(number)s  path: %(path)s  type: %(type)s\n"
            "  name: %(name)s  active: %(active)s  busy: %(busy)s\n"
            "  geometry: %(geometry)r  PedPartition: %(ped)r"
            % {
                "disk": self.disk,
                "fileSystem": self.fileSystem,
                "geometry": self.geometry,
                "number": self.number,
                "path": self.path,
                "type": self.type,
                "name": name,
                "active": self.active,
                "busy": self.busy,
                "ped": self.__partition,
            }
        )
        return s

    def __writeOnly(self, prop):
        raise parted.WriteOnlyProperty(prop)

    @property
    @localeC
    def active(self):
        """True if the partition is active, False otherwise."""
        return bool(self.__partition.is_active())

    @property
    @localeC
    def busy(self):
        """True if the partition is active, False otherwise."""
        return bool(self.__partition.is_busy())

    @property
    def disk(self):
        """The Disk this partition belongs to."""
        return self._disk

    @property
    @localeC
    def path(self):
        """The filesystem path to this partition's device node."""
        return self.__partition.get_path()

    @property
    def number(self):
        """The partition number."""
        return self.__partition.num

    @localeC
    def set_name(self, name):
        """Set the partition name to the given string, on supported labels."""
        self.getPedPartition().set_name(name)

    def get_name(self):
        """The partition name, on supported labels."""
        try:
            return self.__partition.get_name()
        except parted.PartitionException:
            return None

    def set_type_id(self, id):
        """Set the partition type id, as an integer, on supported labels.
        Requires the DISK_TYPE_PARTITION_TYPE_ID flag to be available."""
        if not hasattr(self.getPedPartition(), "set_type_id"):
            raise NotImplementedError("Requires build against parted > 3.5")

        self.getPedPartition().set_type_id(id)

    def get_type_id(self):
        """The partition type uuid, as an integer, on supported labels.
        Requires the DISK_TYPE_PARTITION_TYPE_ID flag to be available."""
        if not hasattr(self.getPedPartition(), "get_type_id"):
            raise NotImplementedError("Requires build against parted > 3.5")

        try:
            return self.__partition.get_type_id()
        except parted.PartitionException:
            return None

    def set_type_uuid(self, uuid):
        """Set the partition type uuid, as 16 bytes, on supported labels.
        Requires the DISK_TYPE_PARTITION_TYPE_UUID flag to be available."""
        if not hasattr(self.getPedPartition(), "set_type_uuid"):
            raise NotImplementedError("Requires build against parted > 3.5")

        self.getPedPartition().set_type_uuid(uuid)

    def get_type_uuid(self):
        """The partition type uuid, as 16 bytes, on supported labels.
        Requires the DISK_TYPE_PARTITION_TYPE_UUID flag to be available."""
        if not hasattr(self.getPedPartition(), "get_type_uuid"):
            raise NotImplementedError("Requires build against parted > 3.5")

        try:
            return self.__partition.get_type_uuid()
        except parted.PartitionException:
            return None

    fileSystem = property(
        lambda s: s._fileSystem, lambda s, v: setattr(s, "_fileSystem", v)
    )
    geometry = property(lambda s: s._geometry, lambda s, v: setattr(s, "_geometry", v))
    system = property(
        lambda s: s.__writeOnly("system"), lambda s, v: s.__partition.set_system(v)
    )
    type = property(
        lambda s: s.__partition.type, lambda s, v: setattr(s.__partition, "type", v)
    )
    name = property(get_name, set_name)
    type_uuid = property(get_type_uuid, set_type_uuid)
    type_id = property(get_type_id, set_type_id)

    @localeC
    def getFlag(self, flag):
        """Get the value of a particular flag on the partition.  Valid flags
        are the _ped.PARTITION_* constants.  See _ped.flag_get_name() and
        _ped.flag_get_by_name() for more help working with partition flags.
        """
        return self.__partition.get_flag(flag)

    @localeC
    def setFlag(self, flag):
        """Set the flag on a partition to the provided value.  On error, a
        PartitionException will be raised.  See getFlag() for more help on
        working with partition flags."""
        return self.__partition.set_flag(flag, 1)

    @localeC
    def unsetFlag(self, flag):
        """Unset the flag on this Partition.  On error, a PartitionException
        will be raised.  See getFlag() for more help on working with
        partition flags."""
        return self.__partition.set_flag(flag, 0)

    @localeC
    def getMaxGeometry(self, constraint):
        """Given a constraint, return the maximum Geometry that self can be
        grown to.  Raises Partitionexception on error."""
        return parted.Geometry(
            PedGeometry=self.disk.getPedDisk().get_max_partition_geometry(
                self.__partition, constraint.getPedConstraint()
            )
        )

    @localeC
    def isFlagAvailable(self, flag):
        """Return True if flag is available on this Partition, False
        otherwise."""
        return self.__partition.is_flag_available(flag)

    @localeC
    def nextPartition(self):
        """Return the Partition following this one on the Disk."""
        partition = self.disk.getPedDisk().next_partition(self.__partition)

        if partition is None:
            return None
        else:
            return parted.Partition(disk=self.disk, PedPartition=partition)

    @localeC
    def getSize(self, unit="MB"):
        """Return the size of the partition in the unit specified.  The unit
        is given as a string corresponding to one of the following
        abbreviations:  b (bytes), KB (kilobytes), MB (megabytes), GB
        (gigabytes), TB (terabytes).  An invalid unit string will raise a
        SyntaxError exception.  The default unit is MB."""
        warnings.warn("use the getLength method", DeprecationWarning)
        return self.geometry.getSize(unit)

    @localeC
    def getLength(self, unit="sectors"):
        """Return the length of the partition in sectors. Optionally, a SI or
        IEC prefix followed by a 'B' may be given in order to convert the
        length into bytes. The allowed values include B, kB, MB, GB, TB, KiB,
        MiB, GiB, and TiB."""
        return self.geometry.getLength(unit)

    def getFlagsAsString(self):
        """Return a comma-separated string representing the flags
        on this partition."""
        flags = []

        for flag in partitionFlag.keys():
            if self.getFlag(flag):
                flags.append(partitionFlag[flag])

        return ", ".join(flags)

    def getMaxAvailableSize(self, unit="MB"):
        """Return the maximum size this Partition can grow to by looking
        at contiguous freespace partitions.  The size is returned in
        the unit specified (default is megabytes).  The unit is a
        string corresponding to one of the following abbreviations:
        b (bytes), KB (kilobytes), MB (megabytes), GB (gigabytes),
        TB (terabytes).  An invalid unit string will raise a
        SyntaxError exception."""
        lunit = unit.lower()

        if lunit not in parted._exponent.keys():
            raise SyntaxError("invalid unit %s given" % (unit))

        maxLength = self.geometry.length
        sectorSize = self.geometry.device.sectorSize

        for partition in self.disk.partitions:
            if partition.type & parted.PARTITION_FREESPACE:
                maxLength += partition.geometry.length
            else:
                break

        return math.floor(maxLength * math.pow(sectorSize, parted._exponent[lunit]))

    def getDeviceNodeName(self):
        """Return the device name for this Partition."""
        return self.path[5:]

    def getPedPartition(self):
        """Return the _ped.Partition object contained in this Partition.
        For internal module use only."""
        return self.__partition

    def resetNumber(self):
        """Reset the partition's number to default"""
        return self.__partition.reset_num()


# collect all partition flags and store them in a hash
partitionFlag = {}
__flag = _ped.partition_flag_next(0)
partitionFlag[__flag] = _ped.partition_flag_get_name(__flag)
__readFlags = True

while __readFlags:
    __flag = _ped.partition_flag_next(__flag)
    if not __flag:
        __readFlags = False
    else:
        partitionFlag[__flag] = _ped.partition_flag_get_name(__flag)



#
# filesystem.py
# Python bindings for libparted (built on top of the _ped Python module).
#
# Copyright The pyparted Project Authors
# SPDX-License-Identifier: GPL-2.0-or-later
#

import _ped
import parted

from parted.decorators import localeC

# XXX: add docstrings!


class FileSystem(object):
    # pylint: disable=W0622
    @localeC
    def __init__(self, type=None, geometry=None, checked=False, PedFileSystem=None):
        if checked:
            c = 1
        else:
            c = 0

        if PedFileSystem is None:
            if type is None:
                raise parted.FileSystemException("no type specified")
            elif geometry is None:
                raise parted.FileSystemException("no geometry specified")

            self._type = type
            self._geometry = geometry
            self._checked = checked
            self.__fileSystem = _ped.FileSystem(
                type=fileSystemType[type], geom=geometry.getPedGeometry(), checked=c
            )
        else:
            self.__fileSystem = PedFileSystem
            self._type = self.__fileSystem.type.name
            self._geometry = parted.Geometry(PedGeometry=self.__fileSystem.geom)

            if self.__fileSystem.checked:
                self._checked = True
            else:
                self._checked = False

    def __eq__(self, other):
        return not self.__ne__(other)

    def __ne__(self, other):
        if not isinstance(self, other.__class__):
            return True

        return self.type != other.type or self.geometry != other.geometry

    def __str__(self):
        s = (
            "parted.FileSystem instance --\n"
            "  type: %(type)s  geometry: %(geometry)r  checked: %(checked)s\n"
            "  PedFileSystem: %(ped)r"
            % {
                "type": self.type,
                "geometry": self.geometry,
                "checked": self.checked,
                "ped": self.__fileSystem,
            }
        )
        return s

    @property
    def type(self):
        """The type of this filesystem, e.g. ext3."""
        return self._type

    @property
    def geometry(self):
        """The Geometry object describing this filesystem."""
        return self._geometry

    @property
    def checked(self):
        """True if this filesystem has been checked, False otherwise."""
        return bool(self._checked)

    def getPedFileSystem(self):
        """Return the _ped.FileSystem object contained in this FileSystem.
        For internal module use only."""
        return self.__fileSystem


# collect all filesystem types and store them in a hash
fileSystemType = {}
__type = _ped.file_system_type_get_next()
fileSystemType[__type.name] = __type

while True:
    try:
        __type = _ped.file_system_type_get_next(__type)
        fileSystemType[__type.name] = __type
    except (IndexError, TypeError, _ped.UnknownTypeException):
        break



#
# Python bindings for libparted (built on top of the _ped Python module).
#
# Copyright The pyparted Project Authors
# SPDX-License-Identifier: GPL-2.0-or-later
#

import sys

if sys.version_info.major >= 3 and sys.version_info.minor >= 3:
    from collections.abc import Sequence
else:
    from collections import Sequence


class CachedList(Sequence):
    """CachedList()

    Provides an immutable list that is constructed from a function that
    could take a while to run.  This is basically the same concept as
    memoization, except that the function does not take any parameters
    and therefore there is nothing to use as a memo.

    The constructor function is provided to __init__, must not take any
    parameters, and must return a list.  The invalidate() method indicates
    that the list is no longer valid and should be reconstucted by
    calling the function again.  It is up to client code to call invalidate.
    The rest of the procedure is handled by this class.

    In all ways, this should appear to be just like a list."""

    def __init__(self, lstFn):
        """Construct a new CachedList.  The lstFn is a function that takes
        no parameters and returns a list.  It will be called lazily - the
        list is not constructed until the first access, which could be
        quite a while after this method is called."""
        self._invalid = True
        self._lst = []
        self._lstFn = lstFn

    def __rebuildList(self):
        if self._invalid:
            self._lst = self._lstFn()
            self._invalid = False

    def __contains__(self, value):
        self.__rebuildList()
        return self._lst.__contains__(value)

    def __getitem__(self, index):
        self.__rebuildList()
        return self._lst.__getitem__(index)

    def __iter__(self):
        self.__rebuildList()
        return self._lst.__iter__()

    def __len__(self):
        self.__rebuildList()
        return len(self._lst)

    def __repr__(self):
        self.__rebuildList()
        return repr(self._lst)

    def __str__(self):
        self.__rebuildList()
        return str(self._lst)

    def __hash__(self):
        return hash(str(self))

    def count(self, value):
        self.__rebuildList()
        return self._lst.count(value)

    def index(self, value, *args, **kwargs):
        self.__rebuildList()
        return self._lst.index(value, *args, **kwargs)

    def invalidate(self):
        """Indicate that the list is no longer valid, due to some external
        changes.  The next access to the list will result in the provided
        list construction function being called to build a new list."""
        self._invalid = True



#
# geometry.py
# Python bindings for libparted (built on top of the _ped Python module).
#
# Copyright The pyparted Project Authors
# SPDX-License-Identifier: GPL-2.0-or-later
#

import parted
import _ped

from parted.decorators import localeC


class Alignment(object):
    """Alignment()

    An Alignment object describes constraints on how sectors and Geometry
    objects are aligned.  Being aligned means that the sector be located at
    a specific sector multiple on a device, or that a geometry must start
    and end at sectors at those specific multiples.  Most methods on this
    object raise ArithmeticError if calculating alignments fails."""

    @localeC
    def __init__(self, *args, **kwargs):
        """Create a new Alignment object from the sectors offset and
        grainSize."""
        if "PedAlignment" in kwargs:
            self.__alignment = kwargs.get("PedAlignment")
        elif "offset" in kwargs and "grainSize" in kwargs:
            self.__alignment = _ped.Alignment(
                kwargs.get("offset"), kwargs.get("grainSize")
            )
        else:
            raise parted.AlignmentException(
                "no offset+grainSize or PedAlignment specified"
            )

    offset = property(
        lambda s: s.__alignment.offset, lambda s, v: setattr(s.__alignment, "offset", v)
    )
    grainSize = property(
        lambda s: s.__alignment.grain_size,
        lambda s, v: setattr(s.__alignment, "grain_size", v),
    )

    def __eq__(self, other):
        return not self.__ne__(other)

    def __ne__(self, other):
        if not isinstance(self, other.__class__):
            return True

        return self.offset != other.offset or self.grainSize != other.grainSize

    def __str__(self):
        s = (
            "parted.Alignment instance --\n"
            "  offset: %(offset)s  grainSize: %(grainSize)s\n"
            "  PedAlignment: %(ped)r"
            % {
                "offset": self.offset,
                "grainSize": self.grainSize,
                "ped": self.__alignment,
            }
        )
        return s

    @localeC
    def intersect(self, b):
        """Create and return a new Alignment that describes the intersection of
        self and alignment b.  A sector will satisfy the new alignment iff
        it satisfies both of the original alignments.  Whether a sector
        satisfies a given alignment is determined by is_aligned()."""
        return parted.Alignment(
            PedAlignment=self.__alignment.intersect(b.getPedAlignment())
        )

    @localeC
    def alignUp(self, geom, sector):
        """Return the closest sector to the provided sector that lies inside
        geom and satisfies the alignment constraint self.  This method
        prefers, but does not guarantee, that the result is beyond sector.
        If no such sector can be found, an ArithmeticError is raised."""
        return self.__alignment.align_up(geom.getPedGeometry(), sector)

    @localeC
    def alignDown(self, geom, sector):
        """Return the closest sector to the provided sector that lies inside
        geom and satisfies the alignment constraint self.  This method
        prefers, but does not guarantee, that the result is below sector.
        If no such sector can be found, an ArithmeticError is raised."""
        return self.__alignment.align_down(geom.getPedGeometry(), sector)

    @localeC
    def alignNearest(self, geom, sector):
        """Return the closest sector to the input sector that lies inside
        geom and satisfies the alignment constraint self.  If no such sector
        can be found, an ArithmeticError is raised."""
        return self.__alignment.align_nearest(geom.getPedGeometry(), sector)

    @localeC
    def isAligned(self, geom, sector):
        """Determine whether sector lies inside geom and satisfies the
        alignment constraint self."""
        if not geom:
            raise TypeError("missing parted.Geometry parameter")

        if sector is None:
            raise TypeError("missing sector parameter")

        return self.__alignment.is_aligned(geom.getPedGeometry(), sector)

    def getPedAlignment(self):
        """Return the _ped.Alignment object contained in this Alignment.
        For internal module use only."""
        return self.__alignment



#
# constraint.py
# Python bindings for libparted (built on top of the _ped Python module).
#
# Copyright The pyparted Project Authors
# SPDX-License-Identifier: GPL-2.0-or-later
#

import parted
import _ped

from parted.decorators import localeC


class Constraint(object):
    """Constraint()

    A Constraint object describes a set of restrictions on other pyparted
    operations.  Constraints can restrict the location and alignment of the
    start and end of a partition, and its minimum and maximum size.  Most
    constraint operations can raise CreateException if creating temporary
    objects fails, or ArithmeticError if an error occurs during
    calculations."""

    @localeC
    def __init__(self, *args, **kwargs):
        """Create a new Constraint object.  There are many different ways to
        create a Constraint, all depending on the parameters passed to
        __init__.  If minGeom and maxGeom are supplied, the constraint will
        be created to satisfy both.  If only one of minGeom or maxGeom are
        supplied, the constraint is only guaranteed to solve the given
        parameter.  If exactGeom is given, the constraint will only be
        satisfied by the given geometry.  If device is given, any region
        on that device will satisfy the constraint.

        If none of the previously mentioned parameters are supplied, all of
        startAlign, EndAlign, startRange, endRange, minSize, and maxSize
        must be given."""
        if "PedConstraint" in kwargs:
            self.__constraint = kwargs.get("PedConstraint")
        elif "minGeom" in kwargs and "maxGeom" in kwargs:
            ming = kwargs.get("minGeom").getPedGeometry()
            maxg = kwargs.get("maxGeom").getPedGeometry()
            self.__constraint = _ped.constraint_new_from_min_max(ming, maxg)
        elif "minGeom" in kwargs:
            ming = kwargs.get("minGeom").getPedGeometry()
            self.__constraint = _ped.constraint_new_from_min(ming)
        elif "maxGeom" in kwargs:
            maxg = kwargs.get("maxGeom").getPedGeometry()
            self.__constraint = _ped.constraint_new_from_max(maxg)
        elif "exactGeom" in kwargs:
            exact = kwargs.get("exactGeom").getPedGeometry()
            self.__constraint = _ped.constraint_exact(exact)
        elif "device" in kwargs:
            dev = kwargs.get("device").getPedDevice()
            self.__constraint = _ped.constraint_any(dev)
        elif (
            "startAlign" in kwargs
            and "endAlign" in kwargs
            and "startRange" in kwargs
            and "endRange" in kwargs
            and "minSize" in kwargs
            and "maxSize" in kwargs
        ):
            starta = kwargs.get("startAlign").getPedAlignment()
            enda = kwargs.get("endAlign").getPedAlignment()
            startr = kwargs.get("startRange").getPedGeometry()
            endr = kwargs.get("endRange").getPedGeometry()
            mins = kwargs.get("minSize")
            maxs = kwargs.get("maxSize")
            self.__constraint = _ped.Constraint(starta, enda, startr, endr, mins, maxs)
        else:
            raise parted.ConstraintException("missing initialization parameters")

    def __eq__(self, other):
        return not self.__ne__(other)

    def __ne__(self, other):
        if hash(self) == hash(other):
            return False

        if not isinstance(self, other.__class__):
            return True

        c1 = self.getPedConstraint()
        c2 = other.getPedConstraint()

        return (
            self.minSize != other.minSize
            or self.maxSize != other.maxSize
            or c1.start_align != c2.start_align
            or c1.end_align != c2.end_align
            or c1.start_range != c2.start_range
            or c1.end_range != c2.end_range
        )

    startAlign = property(
        lambda s: parted.Alignment(PedAlignment=s.__constraint.start_align),
        lambda s, v: setattr(s.__constraint, "start_align", v.getPedAlignment()),
    )

    endAlign = property(
        lambda s: parted.Alignment(PedAlignment=s.__constraint.end_align),
        lambda s, v: setattr(s.__constraint, "end_align", v.getPedAlignment()),
    )

    startRange = property(
        lambda s: parted.Geometry(PedGeometry=s.__constraint.start_range),
        lambda s, v: setattr(s.__constraint, "start_range", v.getPedGeometry()),
    )

    endRange = property(
        lambda s: parted.Geometry(PedGeometry=s.__constraint.end_range),
        lambda s, v: setattr(s.__constraint, "end_range", v.getPedGeometry()),
    )

    minSize = property(
        lambda s: s.__constraint.min_size,
        lambda s, v: setattr(s.__constraint, "min_size", v),
    )

    maxSize = property(
        lambda s: s.__constraint.max_size,
        lambda s, v: setattr(s.__constraint, "max_size", v),
    )

    def __str__(self):
        s = (
            "parted.Constraint instance --\n"
            "  startAlign: %(startAlign)r  endAlign: %(endAlign)r\n"
            "  startRange: %(startRange)r  endRange: %(endRange)r\n"
            "  minSize: %(minSize)s  maxSize: %(maxSize)s\n"
            "  PedConstraint: %(ped)r"
            % {
                "startAlign": self.startAlign,
                "endAlign": self.endAlign,
                "startRange": self.startRange,
                "endRange": self.endRange,
                "minSize": self.minSize,
                "maxSize": self.maxSize,
                "ped": self.__constraint,
            }
        )
        return s

    @localeC
    def intersect(self, b):
        """Return a new constraint that is the intersection of self and the
        provided constraint b.  The returned constraint will therefore be
        more restrictive than either input as it will have to satisfy
        both."""
        return parted.Constraint(
            PedConstraint=self.__constraint.intersect(b.getPedConstraint())
        )

    @localeC
    def solveMax(self):
        """Return a new geometry that is the largest region satisfying self.
        There may be more than one solution, and there are no guarantees as
        to which solution will be returned."""
        return parted.Geometry(PedGeometry=self.__constraint.solve_max())

    @localeC
    def solveNearest(self, geom):
        """Return a new geometry that is the nearest region to geom that
        satisfies self.  This function does not guarantee any specific
        meaning of 'nearest'."""
        return parted.Geometry(
            PedGeometry=self.__constraint.solve_nearest(geom.getPedGeometry())
        )

    @localeC
    def isSolution(self, geom):
        """Does geom satisfy this constraint?"""
        return self.__constraint.is_solution(geom.getPedGeometry())

    def getPedConstraint(self):
        """Return the _ped.Constraint object contained in this Constraint.
        For internal module use only."""
        return self.__constraint



#
# __init__.py
# Python bindings for libparted (built on top of the _ped Python module).
#
# Copyright The pyparted Project Authors
# SPDX-License-Identifier: GPL-2.0-or-later
#

from __future__ import division

import math
import platform
import re
import sys
import warnings
import _ped

__all__ = [
    "Alignment",
    "Constraint",
    "Device",
    "Disk",
    "FileSystem",
    "Geometry",
    "Partition",
]

from _ped import AlignmentException
from _ped import CreateException
from _ped import ConstraintException
from _ped import DeviceException
from _ped import DiskException
from _ped import DiskLabelException
from _ped import FileSystemException
from _ped import GeometryException
from _ped import IOException
from _ped import NotNeededException
from _ped import PartedException
from _ped import PartitionException
from _ped import TimerException
from _ped import UnknownDeviceException
from _ped import UnknownTypeException

from _ped import register_exn_handler
from _ped import clear_exn_handler

from parted.alignment import Alignment
from parted.constraint import Constraint
from parted.device import Device
from parted.disk import Disk
from parted.disk import diskType
from parted.disk import diskFlag
from parted.filesystem import FileSystem
from parted.filesystem import fileSystemType
from parted.geometry import Geometry
from parted.partition import Partition
from parted.partition import partitionFlag

# the enumerated types in _ped need to be available from here too
from _ped import UNIT_SECTOR
from _ped import UNIT_BYTE
from _ped import UNIT_KILOBYTE
from _ped import UNIT_MEGABYTE
from _ped import UNIT_GIGABYTE
from _ped import UNIT_TERABYTE
from _ped import UNIT_COMPACT
from _ped import UNIT_CYLINDER
from _ped import UNIT_CHS
from _ped import UNIT_PERCENT
from _ped import UNIT_KIBIBYTE
from _ped import UNIT_MEBIBYTE
from _ped import UNIT_GIBIBYTE
from _ped import UNIT_TEBIBYTE

units = {}
units[UNIT_SECTOR] = "sector"
units[UNIT_BYTE] = "byte"
units[UNIT_KILOBYTE] = "kilobyte"
units[UNIT_MEGABYTE] = "megabyte"
units[UNIT_GIGABYTE] = "gigabyte"
units[UNIT_TERABYTE] = "terabyte"
units[UNIT_COMPACT] = "compact"
units[UNIT_CYLINDER] = "cylinder"
units[UNIT_CHS] = "chs"
units[UNIT_PERCENT] = "percent"
units[UNIT_KIBIBYTE] = "kibibyte"
units[UNIT_MEBIBYTE] = "mebibyte"
units[UNIT_GIBIBYTE] = "gibibyte"
units[UNIT_TEBIBYTE] = "tebibyte"

from _ped import DEVICE_UNKNOWN
from _ped import DEVICE_SCSI
from _ped import DEVICE_IDE
from _ped import DEVICE_DAC960
from _ped import DEVICE_CPQARRAY
from _ped import DEVICE_FILE
from _ped import DEVICE_ATARAID
from _ped import DEVICE_I2O
from _ped import DEVICE_UBD
from _ped import DEVICE_DASD
from _ped import DEVICE_VIODASD
from _ped import DEVICE_SX8
from _ped import DEVICE_DM
from _ped import DEVICE_XVD
from _ped import DEVICE_SDMMC
from _ped import DEVICE_VIRTBLK
from _ped import DEVICE_NVME

devices = {}
devices[DEVICE_UNKNOWN] = "unknown"
devices[DEVICE_SCSI] = "scsi"
devices[DEVICE_IDE] = "ide"
devices[DEVICE_DAC960] = "dac960"
devices[DEVICE_CPQARRAY] = "cpqarray"
devices[DEVICE_FILE] = "file"
devices[DEVICE_ATARAID] = "ataraid"
devices[DEVICE_I2O] = "i2o"
devices[DEVICE_UBD] = "ubd"
devices[DEVICE_DASD] = "dasd"
devices[DEVICE_VIODASD] = "viodasd"
devices[DEVICE_SX8] = "sx8"
devices[DEVICE_DM] = "dm"
devices[DEVICE_XVD] = "xvd"
devices[DEVICE_SDMMC] = "sdmmc"
devices[DEVICE_VIRTBLK] = "virtblk"
devices[DEVICE_NVME] = "nvme"

from _ped import PARTITION_NORMAL
from _ped import PARTITION_LOGICAL
from _ped import PARTITION_EXTENDED
from _ped import PARTITION_FREESPACE
from _ped import PARTITION_METADATA
from _ped import PARTITION_PROTECTED

partitions = {}
partitions[PARTITION_NORMAL] = "normal"
partitions[PARTITION_LOGICAL] = "logical"
partitions[PARTITION_EXTENDED] = "extended"
partitions[PARTITION_FREESPACE] = "freespace"
partitions[PARTITION_METADATA] = "metadata"
partitions[PARTITION_PROTECTED] = "protected"

from _ped import PARTITION_BOOT
from _ped import PARTITION_ROOT
from _ped import PARTITION_SWAP
from _ped import PARTITION_HIDDEN
from _ped import PARTITION_RAID
from _ped import PARTITION_LVM
from _ped import PARTITION_LBA
from _ped import PARTITION_HPSERVICE
from _ped import PARTITION_PALO
from _ped import PARTITION_PREP
from _ped import PARTITION_MSFT_RESERVED
from _ped import PARTITION_APPLE_TV_RECOVERY
from _ped import PARTITION_BIOS_GRUB
from _ped import PARTITION_DIAG
from _ped import PARTITION_LEGACY_BOOT

partitions[PARTITION_BOOT] = "boot"
partitions[PARTITION_ROOT] = "root"
partitions[PARTITION_SWAP] = "swap"
partitions[PARTITION_HIDDEN] = "hidden"
partitions[PARTITION_RAID] = "raid"
partitions[PARTITION_LVM] = "lvm"
partitions[PARTITION_LBA] = "lba"
partitions[PARTITION_HPSERVICE] = "hpservice"
partitions[PARTITION_PALO] = "palo"
partitions[PARTITION_PREP] = "prep"
partitions[PARTITION_MSFT_RESERVED] = "msft_reserved"
partitions[PARTITION_APPLE_TV_RECOVERY] = "apple_tv_recovery"
partitions[PARTITION_BIOS_GRUB] = "bios_grub"
partitions[PARTITION_DIAG] = "diag"
partitions[PARTITION_LEGACY_BOOT] = "legacy_boot"

if hasattr(_ped, "PARTITION_MSFT_DATA"):
    # pylint: disable=E0611
    from _ped import PARTITION_MSFT_DATA

    partitions[PARTITION_MSFT_DATA] = "msft_data"
if hasattr(_ped, "PARTITION_IRST"):
    # pylint: disable=E0611
    from _ped import PARTITION_IRST

    partitions[PARTITION_IRST] = "irst"
if hasattr(_ped, "PARTITION_ESP"):
    # pylint: disable=E0611
    from _ped import PARTITION_ESP

    partitions[PARTITION_ESP] = "esp"
if hasattr(_ped, "PARTITION_NONFS"):
    # pylint: disable=E0611
    from _ped import PARTITION_NONFS

    partitions[PARTITION_NONFS] = "nonfs"
if hasattr(_ped, "PARTITION_CHROMEOS_KERNEL"):
    # pylint: disable=E0611
    from _ped import PARTITION_CHROMEOS_KERNEL

    partitions[PARTITION_CHROMEOS_KERNEL] = "chromeos_kernel"
if hasattr(_ped, "PARTITION_BLS_BOOT"):
    # pylint: disable=E0611
    from _ped import PARTITION_BLS_BOOT

    partitions[PARTITION_BLS_BOOT] = "bls_boot"
if hasattr(_ped, "PARTITION_LINUX_HOME"):
    # pylint: disable=E0611
    from _ped import PARTITION_LINUX_HOME

    partitions[PARTITION_LINUX_HOME] = "linux-home"
if hasattr(_ped, "PARTITION_NO_AUTOMOUNT"):
    # pylint: disable=E0611
    from _ped import PARTITION_NO_AUTOMOUNT

    partitions[PARTITION_NO_AUTOMOUNT] = "no_automount"

from _ped import DISK_CYLINDER_ALIGNMENT
from _ped import DISK_GPT_PMBR_BOOT

from _ped import DISK_TYPE_EXTENDED
from _ped import DISK_TYPE_PARTITION_NAME

if hasattr(_ped, "DISK_TYPE_PARTITION_TYPE_ID"):
    # pylint: disable=E0611
    from _ped import DISK_TYPE_PARTITION_TYPE_ID

if hasattr(_ped, "DISK_TYPE_PARTITION_TYPE_UUID"):
    # pylint: disable=E0611
    from _ped import DISK_TYPE_PARTITION_TYPE_UUID

if hasattr(_ped, "DISK_TYPE_DISK_UUID"):
    # pylint: disable=E0611
    from _ped import DISK_TYPE_DISK_UUID

if hasattr(_ped, "DISK_TYPE_PARTITION_UUID"):
    # pylint: disable=E0611
    from _ped import DISK_TYPE_PARTITION_UUID

from _ped import EXCEPTION_TYPE_INFORMATION
from _ped import EXCEPTION_TYPE_WARNING
from _ped import EXCEPTION_TYPE_ERROR
from _ped import EXCEPTION_TYPE_FATAL
from _ped import EXCEPTION_TYPE_BUG
from _ped import EXCEPTION_TYPE_NO_FEATURE

from _ped import EXCEPTION_RESOLVE_UNHANDLED
from _ped import EXCEPTION_RESOLVE_FIX
from _ped import EXCEPTION_RESOLVE_YES
from _ped import EXCEPTION_RESOLVE_NO
from _ped import EXCEPTION_RESOLVE_OK
from _ped import EXCEPTION_RESOLVE_RETRY
from _ped import EXCEPTION_RESOLVE_IGNORE
from _ped import EXCEPTION_RESOLVE_CANCEL

from _ped import EXCEPTION_OPT_OK_CANCEL
from _ped import EXCEPTION_OPT_YES_NO
from _ped import EXCEPTION_OPT_YES_NO_CANCEL
from _ped import EXCEPTION_OPT_IGNORE_CANCEL
from _ped import EXCEPTION_OPT_RETRY_CANCEL
from _ped import EXCEPTION_OPT_RETRY_IGNORE_CANCEL

from parted.decorators import localeC

if sys.version_info >= (3,):
    string_types = str
else:
    string_types = basestring  # pylint: disable=undefined-variable

partitionTypesDict = {
    0x00: "Empty",
    0x01: "DOS 12-bit FAT",
    0x02: "XENIX root",
    0x03: "XENIX usr",
    0x04: "DOS 16-bit <32M",
    0x05: "Extended",
    0x06: "DOS 16-bit >=32M",
    0x07: "NTFS/HPFS",
    0x08: "AIX",
    0x09: "AIX bootable",
    0x0A: "OS/2 Boot Manager",
    0x0B: "Win95 FAT32",
    0x0C: "Win95 FAT32",
    0x0E: "Win95 FAT16",
    0x0F: "Win95 Ext'd",
    0x10: "OPUS",
    0x11: "Hidden FAT12",
    0x12: "Compaq Setup",
    0x14: "Hidden FAT16 <32M",
    0x16: "Hidden FAT16",
    0x17: "Hidden HPFS/NTFS",
    0x18: "AST SmartSleep",
    0x1B: "Hidden Win95 FAT32",
    0x1C: "Hidden Win95 FAT32 (LBA)",
    0x1E: "Hidden Win95 FAT16 (LBA)",
    0x24: "NEC_DOS",
    0x39: "Plan 9",
    0x40: "Venix 80286",
    0x41: "PPC_PReP Boot",
    0x42: "SFS",
    0x4D: "QNX4.x",
    0x4E: "QNX4.x 2nd part",
    0x4F: "QNX4.x 2nd part",
    0x51: "Novell?",
    0x52: "Microport",
    0x63: "GNU HURD",
    0x64: "Novell Netware 286",
    0x65: "Novell Netware 386",
    0x75: "PC/IX",
    0x80: "Old MINIX",
    0x81: "Linux/MINIX",
    0x82: "Linux swap",
    0x83: "Linux native",
    0x84: "OS/2 hidden C:",
    0x85: "Linux Extended",
    0x86: "NTFS volume set",
    0x87: "NTFS volume set",
    0x8E: "Linux LVM",
    0x93: "Amoeba",
    0x94: "Amoeba BBT",
    0x9F: "BSD/OS",
    0xA0: "IBM Thinkpad hibernation",
    0xA5: "BSD/386",
    0xA6: "OpenBSD",
    0xB7: "BSDI fs",
    0xB8: "BSDI swap",
    0xBF: "Solaris",
    0xC7: "Syrinx",
    0xDB: "CP/M",
    0xDE: "Dell Utility",
    0xE1: "DOS access",
    0xE3: "DOS R/O",
    0xEB: "BEOS",
    0xEE: "EFI GPT",
    0xEF: "EFI (FAT-12/16/32)",
    0xF2: "DOS secondary",
    0xFD: "Linux RAID",
    0xFF: "BBT",
}

# Exponents for 1024 used when converting sizes to byte-sized
# units for display.  The keys are:
#   b     bytes       1024^0 = 1
#   kb    kilobytes   1024^1 = 1024
#   mb    megabytes   1024^2 = 1048576
#   gb    gigabytes   1024^3 = 1073741824
#   tb    terabytes   1024^4 = 1099511627776
#   pb    petabytes   1024^5 = 1125899906842624
#   eb    exabytes    1024^6 = 1152921504606846976
#   zb    zettabytes  1024^7 = 1180591620717411303424
#   yb    yottabytes  1024^8 = 1208925819614629174706176
# The resulting value for 1024 raised to the power is used as
# the divisor for conversion functions.
_exponent = {
    "b": 0,
    "kb": 1,
    "mb": 2,
    "gb": 3,
    "tb": 4,
    "pb": 5,
    "eb": 6,
    "zb": 7,
    "yb": 8,
}

# Refercences:
#
# 1. NIST Special Publication 330, 2008 Edition, Barry N. Taylor and Ambler
#    Thompson, Editors
#    The International System of Units (SI)
#    Available from: http://physics.nist.gov/cuu/pdf/sp811.pdf
#
# 2. International standard IEC 60027-2, third edition,
#    Letter symbols to be used in electrical technology --
#    Part 2: Telecommunications and electronics.
#
# See the links below for quick online summaries:
#
# SI units:  http://physics.nist.gov/cuu/Units/prefixes.html
# IEC units: http://physics.nist.gov/cuu/Units/binary.html
__exponents = {
    "B": 1,  # byte
    "kB": 1000**1,  # kilobyte
    "MB": 1000**2,  # megabyte
    "GB": 1000**3,  # gigabyte
    "TB": 1000**4,  # terabyte
    "PB": 1000**5,  # petabyte
    "EB": 1000**6,  # exabyte
    "ZB": 1000**7,  # zettabyte
    "YB": 1000**8,  # yottabyte
    "KiB": 1024**1,  # kibibyte
    "MiB": 1024**2,  # mebibyte
    "GiB": 1024**3,  # gibibyte
    "TiB": 1024**4,  # tebibyte
    "PiB": 1024**5,  # pebibyte
    "EiB": 1024**6,  # exbibyte
    "ZiB": 1024**7,  # zebibyte
    "YiB": 1024**8,  # yobibyte
}


def formatBytes(bytes_, unit):
    """Convert bytes_ using an SI or IEC prefix. Note that unit is a
    case sensitive string that must exactly match one of the IEC or SI
    prefixes followed by 'B' (e.g. 'GB')."""

    if unit not in __exponents.keys():
        raise SyntaxError("{:} is not a valid SI or IEC byte unit".format(unit))
    else:
        return bytes_ / __exponents[unit]


def sizeToSectors(bytes_, unit, sector_size):
    """Convert bytes_ of unit to a number of sectors. Note that unit is a
    case sensitive string that must exactly match one of the IEC or SI
    prefixes followed by 'B' (e.g. 'GB')."""

    if unit not in __exponents.keys():
        raise SyntaxError("{:} is not a valid SI or IEC byte unit".format(unit))
    else:
        return math.ceil(bytes_ * __exponents[unit] / sector_size)


# Valid disk labels per architecture type.  The list of label
# names map to keys in the parted.diskType hash table.
archLabels = {
    "i386": ["msdos", "gpt"],
    "s390": ["dasd", "msdos"],
    "alpha": ["bsd", "msdos"],
    "sparc": ["sun"],
    "ia64": ["msdos", "gpt"],
    "ppc": ["msdos", "mac", "amiga", "gpt"],
    "ppc64": ["msdos", "mac", "amiga", "gpt"],
    "ppc64le": ["msdos", "gpt"],
    "x86_64": ["msdos", "gpt"],
    "aarch64": ["msdos", "gpt"],
    "mips64": ["msdos", "gpt"],
    "armv7l": ["msdos", "gpt"],
    "riscv32": ["msdos", "gpt"],
    "riscv64": ["msdos", "gpt"],
    "loongarch32": ["msdos", "gpt"],
    "loongarch64": ["msdos", "gpt"],
}

# Adapted from:
# http://stackoverflow.com/questions/922550/how-to-mark-a-global-as-deprecated-in-python
#
# Remember that DeprecationWarnings are ignored by default as they are not really
# useful to users.  Developers can turn on DeprecationWarning notices by passing
# the -Wd option to python or by setting PYTHONWARNINGS=d in the environment.
def Deprecated(mod, deprecated=None):
    """Return a wrapped object that warns about deprecated accesses."""

    if not deprecated:
        deprecated = {}

    class Wrapper(object):
        warnmsg = "%s is deprecated and will be removed in a future release."

        def __getattr__(self, attr):
            if attr in deprecated.keys():
                msg = self.warnmsg + " " + deprecated[attr]
                warnings.warn(msg % attr, DeprecationWarning)

            return getattr(mod, attr)

        def __setattr__(self, attr, value):
            if attr in deprecated.keys():
                msg = self.warnmsg + " " + deprecated[attr]
                warnings.warn(msg % attr, DeprecationWarning)
            setattr(mod, attr, value)

    return Wrapper()


# Valid disk labels and their applicable architectures.  The label names map
# to keys in the parted.diskType hash table.
__archLabels = (
    ("amiga", "ppc(64)?$"),
    ("bsd", "alpha$"),
    ("dasd", "s390x?$"),
    (
        "gpt",
        "i[3-6]86$|x86_64$|ia64$|ppc(64|64le)?$|aarch64$|armv7l$|riscv(32|64)$|loongarch(32|64)$",
    ),
    ("mac", "ppc(64)?$"),
    (
        "msdos",
        "i[3-6]86$|x86_64$|s390x?$|alpha$|ia64$|ppc(64|64le)?$|aarch64$|mips64$|armv7l$|riscv(32|64)$|loongarch(32|64)$",
    ),
    ("sun", "sparc(64)?$"),
)


def getLabels(arch=None):
    """Return a set containing the disk labels compatible with the
    architecture of the computer calling this function. If an architecture
    is passed, return the labels compatible with that architecture."""
    labels = set()
    if arch is None:
        arch = platform.machine()

    for label, regex in __archLabels:
        if re.match(regex, arch):
            labels.add(label)

    return labels


class ReadOnlyProperty(Exception):
    """Exception raised when a write operation occurs on a read-only property."""

    # pylint: disable=W0231
    def __init__(self, prop=""):
        self.message = "%s is a read-only property" % (prop,)


class WriteOnlyProperty(Exception):
    """Exception raised when a read operation occurs on a write-only property."""

    # pylint: disable=W0231
    def __init__(self, prop=""):
        self.message = "%s is a write-only property" % (prop,)


@localeC
def getDevice(path):
    """Given the operating system level path to a device node, return
    a Device object for that disk.  Raises DeviceException if an invalid
    path is given."""
    return Device(path=path)


@localeC
def getAllDevices():
    """Return a list of Device objects for all devices in the system."""
    from _ped import device_probe_all
    from _ped import device_get_next

    lst = []
    device = None

    device_probe_all()

    while True:
        try:
            if not device:
                device = device_get_next()
            else:
                device = device_get_next(device)

            lst.append(Device(PedDevice=device))
        except IndexError:
            return lst


@localeC
def freeAllDevices():
    """Free all Device objects.  There is no reason to call this function."""
    from _ped import device_free_all

    return device_free_all()


@localeC
def probeForSpecificFileSystem(fstype, geometry):
    """Call the _ped.file_system_probe_specific() function given the
    filesystem type and geometry.  fstype must be a string
    representing a valid _ped.FileSystemType, geometry is a
    parted.Geometry."""
    from _ped import file_system_probe_specific

    geom = file_system_probe_specific(fileSystemType[fstype], geometry.getPedGeometry())
    return geometry.Geometry(PedGeometry=geom)


@localeC
def probeFileSystem(geometry):
    """Return the name of the filesystem detected on the given
    Geometry.  Returns None is no filesystem found."""
    from _ped import file_system_probe

    fstype = file_system_probe(geometry.getPedGeometry())
    return fstype.name


@localeC
def freshDisk(device, ty):
    """Return a Disk object for this Device and using this DiskType.
    The type should be a member of the parted.diskType hash,
    either a key or a value.

    The new label is not written to disk until commitToDevice()
    is called on the Disk."""
    from _ped import disk_new_fresh, DiskType

    if isinstance(ty, string_types):
        ty = diskType[ty]
    elif not isinstance(ty, DiskType):
        raise TypeError("type must be a key or value in parted.diskType", ty)

    peddisk = disk_new_fresh(device.getPedDevice(), ty)
    return Disk(PedDisk=peddisk)


@localeC
def newDisk(device):
    """Return a Disk object for this Device. Read the partition table off
    a device (if one is found)."""
    from _ped import disk_new

    peddisk = disk_new(device.getPedDevice())
    return Disk(PedDisk=peddisk)


@localeC
def version():
    """Return a dict containing the pyparted and libparted versions."""
    from _ped import libparted_version
    from _ped import pyparted_version

    ver = {}
    ver["libparted"] = libparted_version()
    ver["pyparted"] = pyparted_version()
    return ver


# Mark deprecated items
_deprecated = {
    "partitionTypesDict": "DOS disk label types are not provided "
    "by libparted, so the codes are not "
    "useful.",
    "_exponent": "Use __exponents instead.",
    "archLabels": "Use getLabels() instead.",
}
sys.modules[__name__] = Deprecated(sys.modules[__name__], _deprecated)



#
# disk.py
# Python bindings for libparted (built on top of the _ped Python module).
#
# Copyright The pyparted Project Authors
# SPDX-License-Identifier: GPL-2.0-or-later
#

import _ped
import parted

from parted.cachedlist import CachedList
from parted.decorators import localeC


class Disk(object):
    """Disk()

    A Disk object describes a type of device in the system.  Disks
    can hold partitions.  A Disk is a basic operating system-specific
    object."""

    @localeC
    def __init__(self, device=None, PedDisk=None):
        """Create a new Disk object from the device and type specified.  The
        device is a Device object and type is a string matching a key in
        the diskType hash."""
        if PedDisk:
            self.__disk = PedDisk

            if device is None:
                self._device = parted.Device(PedDevice=self.__disk.dev)
            else:
                self._device = device
        elif device is None:
            raise parted.DiskException("no device specified")
        else:
            self.__disk = _ped.Disk(device.getPedDevice())
            self._device = device

        # pylint: disable=W0108
        self._partitions = CachedList(lambda: self.__getPartitions())

    def _hasSameParts(self, other):
        if len(self.partitions) != len(other.partitions):
            return False

        partIter = zip(self.partitions, other.partitions)
        while True:
            try:
                (left, right) = next(partIter)
                if left != right:
                    return False
            except StopIteration:
                return True

    def __eq__(self, other):
        return not self.__ne__(other)

    def __ne__(self, other):
        if not isinstance(self, other.__class__):
            return True

        return self.device != other.device or not self._hasSameParts(other)

    def __str__(self):
        s = (
            "parted.Disk instance --\n"
            "  type: %(type)s  primaryPartitionCount: %(primaryCount)s\n"
            "  lastPartitionNumber: %(last)s  maxPrimaryPartitionCount: %(max)s\n"
            "  partitions: %(partitions)s\n"
            "  device: %(device)r\n"
            "  PedDisk: %(ped)r"
            % {
                "type": self.type,
                "primaryCount": self.primaryPartitionCount,
                "last": self.lastPartitionNumber,
                "max": self.maxPrimaryPartitionCount,
                "partitions": self.partitions,
                "device": self.device,
                "ped": self.__disk,
            }
        )
        return s

    def __getPartitions(self):
        """Construct a list of partitions on the disk.  This is called only as
        needed from the self.partitions property, which just happens to be
        a CachedList."""
        partitions = []
        partition = self.getFirstPartition()

        while partition:
            if (
                partition.type & parted.PARTITION_FREESPACE
                or partition.type & parted.PARTITION_METADATA
                or partition.type & parted.PARTITION_PROTECTED
            ):
                partition = partition.nextPartition()
                continue

            partitions.append(partition)
            partition = partition.nextPartition()

        return partitions

    @property
    @localeC
    def primaryPartitionCount(self):
        """The number of primary partitions on this disk."""
        return self.__disk.get_primary_partition_count()

    @property
    @localeC
    def lastPartitionNumber(self):
        """The last assigned partition number currently on this disk."""
        return self.__disk.get_last_partition_num()

    @property
    @localeC
    def maxPrimaryPartitionCount(self):
        """The maximum number of primary partitions allowed on this disk."""
        return self.__disk.get_max_primary_partition_count()

    @property
    @localeC
    def maxSupportedPartitionCount(self):
        """The maximum number of partitions allowed on this disk."""
        return self.__disk.get_max_supported_partition_count()

    @property
    @localeC
    def partitionAlignment(self):
        """Partition start address Alignment."""
        alignment = self.__disk.get_partition_alignment()
        return parted.Alignment(PedAlignment=alignment)

    @property
    @localeC
    def maxPartitionLength(self):
        """Max Partition Length the disk's label can represent."""
        return self.__disk.max_partition_length()

    @property
    @localeC
    def maxPartitionStartSector(self):
        """Max Partition Start Sector the disk's label can represent."""
        return self.__disk.max_partition_start_sector()

    @localeC
    def getFlag(self, flag):
        """Get the value of a particular flag on the disk.  Valid flags
        are the _ped.DISK_* constants.  See _ped.disk_flag_get_name() and
        _ped.disk_flag_get_by_name() for more help working with disk flags.
        """
        return self.__disk.get_flag(flag)

    @localeC
    def setFlag(self, flag):
        """Set the flag on this disk.  On error, an Exception will be raised.
        See getFlag() for more help on working with disk flags."""
        return self.__disk.set_flag(flag, 1)

    @localeC
    def unsetFlag(self, flag):
        """Unset the flag on this disk.  On error, an Exception will be raised.
        See getFlag() for more help on working with disk flags."""
        return self.__disk.set_flag(flag, 0)

    @localeC
    def isFlagAvailable(self, flag):
        """Return True if flag is available on this Disk, False
        otherwise."""
        return self.__disk.is_flag_available(flag)

    @property
    def partitions(self):
        """The list of partitions currently on this disk."""
        return self._partitions

    @property
    def device(self):
        """The underlying Device holding this disk and partitions."""
        return self._device

    type = property(
        lambda s: s.__disk.type.name,
        lambda s, v: setattr(s.__disk, "type", parted.diskType[v]),
    )

    @localeC
    def duplicate(self):
        """Make a deep copy of this Disk."""
        return Disk(PedDisk=self.__disk.duplicate())

    @localeC
    def destroy(self):
        """Closes the Disk ensuring all outstanding writes are flushed."""
        return self.__disk.destroy()

    @localeC
    def commit(self):
        """Writes in-memory changes to a partition table to disk and
        informs the operating system of the changes.  Equivalent to
        calling self.commitToDevice() then self.commitToOS()."""
        self.partitions.invalidate()

        return self.__disk.commit()

    @localeC
    def commitToDevice(self):
        """Write the changes made to the in-memory description of a
        partition table to the device."""
        self.partitions.invalidate()

        return self.__disk.commit_to_dev()

    @localeC
    def commitToOS(self):
        """Tell the operating system kernel about the partition table
        layout of this Disk."""
        self.partitions.invalidate()

        return self.__disk.commit_to_os()

    @localeC
    def check(self):
        """Perform a sanity check on the partition table of this Disk."""
        return self.__disk.check()

    @localeC
    def supportsFeature(self, feature):
        """Check that the disk type supports the provided feature."""
        return self.__disk.type.check_feature(feature)

    @localeC
    def addPartition(self, partition=None, constraint=None):
        """Add a new Partition to this Disk with the given Constraint."""
        if constraint:
            result = self.__disk.add_partition(
                partition.getPedPartition(), constraint.getPedConstraint()
            )
        elif not partition:
            raise parted.DiskException("no partition or constraint specified")
        else:
            result = self.__disk.add_partition(partition.getPedPartition())

        if result:
            partition.geometry = parted.Geometry(
                PedGeometry=partition.getPedPartition().geom
            )
            self.partitions.invalidate()
            return True
        else:
            return False

    @localeC
    def removePartition(self, partition=None):
        """Removes specified Partition from this Disk.  NOTE:  If the
        Partition is an extended partition, it must not contain any
        logical partitions.  Also note that the partition is not
        actually destroyed unless you use the deletePartition()
        method."""
        if not partition:
            raise parted.DiskException("no partition specified")

        if self.__disk.remove_partition(partition.getPedPartition()):
            self.partitions.invalidate()
            return True
        else:
            return False

    @localeC
    def deletePartition(self, partition):
        """Removes specified Partition from this Disk under the same
        conditions as removePartition(), but also destroy the
        removed Partition."""
        if self.__disk.delete_partition(partition.getPedPartition()):
            self.partitions.invalidate()
            return True
        else:
            return False

    @localeC
    def deleteAllPartitions(self):
        """Removes and destroys all Partitions in this Disk."""
        if self.__disk.delete_all():
            self.partitions.invalidate()
            return True
        else:
            return False

    @localeC
    def setPartitionGeometry(
        self, partition=None, constraint=None, start=None, end=None
    ):
        """Sets the Geometry of the specified Partition using the given
        Constraint and start and end sectors.  Note that this method
        does not modify the partition contents, just the partition
        table."""
        if not partition or not constraint:
            raise parted.DiskException("no partition or constraint specified")

        if not start or not end:
            raise parted.DiskException("no start or end geometry specified")

        return self.__disk.set_partition_geom(
            partition.getPedPartition(), constraint.getPedConstraint(), start, end
        )

    @localeC
    def maximizePartition(self, partition=None, constraint=None):
        """Grow the Partition's Geometry to the maximum possible subject
        to Constraint."""
        if not partition:
            raise parted.DiskException("no partition specified")

        if constraint:
            return self.__disk.maximize_partition(
                partition.getPedPartition(), constraint.getPedConstraint()
            )
        else:
            return self.__disk.maximize_partition(partition.getPedPartition())

    @localeC
    def calculateMaxPartitionGeometry(self, partition=None, constraint=None):
        """Get the maximum Geometry the Partition can be grown to,
        subject to the given Constraint."""
        if not partition:
            raise parted.DiskException("no partition specified")

        if constraint:
            return parted.Geometry(
                PedGeometry=self.__disk.get_max_partition_geometry(
                    partition.getPedPartition(), constraint.getPedConstraint()
                )
            )
        else:
            return parted.Geometry(
                PedGeometry=self.__disk.get_max_partition_geometry(
                    partition.getPedPartition()
                )
            )

    @localeC
    def minimizeExtendedPartition(self):
        """Reduce the size of the extended partition to a minimum while
        still wrapping its logical partitions.  If there are no logical
        partitions, remove the extended partition."""
        ret = self.__disk.minimize_extended_partition()

        if ret:
            self.partitions.invalidate()

        return ret

    @localeC
    def getPartitionBySector(self, sector):
        """Returns the Partition that contains the sector.  If the sector
        lies within a logical partition, then the logical partition is
        returned (not the extended partition)."""
        return parted.Partition(
            disk=self, PedPartition=self.__disk.get_partition_by_sector(sector)
        )

    @localeC
    def getExtendedPartition(self):
        """Return the extended Partition, if any, on this Disk."""
        try:
            return parted.Partition(
                disk=self, PedPartition=self.__disk.extended_partition()
            )
        # pylint: disable=bare-except
        except:
            return None

    def __filterPartitions(self, fn):
        return [part for part in self.partitions if fn(part)]

    def getLogicalPartitions(self):
        """Return a list of logical Partitions on this Disk."""
        return self.__filterPartitions(
            lambda p: p.active and p.type & parted.PARTITION_LOGICAL
        )

    def getPrimaryPartitions(self):
        """Return a list of primary (or normal) Partitions on this Disk."""
        return self.__filterPartitions(lambda p: p.type == parted.PARTITION_NORMAL)

    def getRaidPartitions(self):
        """Return a list of RAID (or normal) Partitions on this Disk."""
        return self.__filterPartitions(
            lambda p: p.active and p.getFlag(parted.PARTITION_RAID)
        )

    def getLVMPartitions(self):
        """Return a list of physical volume-type Partitions on this Disk."""
        return self.__filterPartitions(
            lambda p: p.active and p.getFlag(parted.PARTITION_LVM)
        )

    @localeC
    def getFreeSpaceRegions(self):
        """Return a list of Geometry objects representing the available
        free space regions on this Disk."""
        freespace = []
        part = self.__disk.next_partition()

        while part:
            if part.type & parted.PARTITION_FREESPACE:
                freespace.append(parted.Geometry(PedGeometry=part.geom))

            part = self.__disk.next_partition(part)

        return freespace

    @localeC
    def getFreeSpacePartitions(self):
        """Return a list of Partition objects representing the available
        free space regions on this Disk."""
        freespace = []
        part = self.__disk.next_partition()

        while part:
            if part.type & parted.PARTITION_FREESPACE:
                freespace.append(parted.Partition(disk=self, PedPartition=part))

            part = self.__disk.next_partition(part)

        return freespace

    @localeC
    def getFirstPartition(self):
        """Return the first Partition object on the disk or None if
        there is not one."""
        return parted.Partition(disk=self, PedPartition=self.__disk.next_partition())

    @localeC
    def getPartitionByPath(self, path):
        """Return a Partition object associated with the partition device
        path, such as /dev/sda1.  Returns None if no partition is found."""
        for partition in self.partitions:
            if partition.path == path:
                return partition

        return None

    def getPedDisk(self):
        """Return the _ped.Disk object contained in this Disk.  For internal
        module use only."""
        return self.__disk


# collect all disk types and store them in a hash
diskType = {}
__type = _ped.disk_type_get_next()
diskType[__type.name] = __type

while True:
    try:
        __type = _ped.disk_type_get_next(__type)
        diskType[__type.name] = __type
    except (IndexError, TypeError, _ped.UnknownTypeException):
        break

# collect all disk flags and store them in a hash
diskFlag = {}
__flag = _ped.disk_flag_next(0)
diskFlag[__flag] = _ped.disk_flag_get_name(__flag)
__readFlags = True

while __readFlags:
    __flag = _ped.disk_flag_next(__flag)
    if not __flag:
        __readFlags = False
    else:
        diskFlag[__flag] = _ped.disk_flag_get_name(__flag)



#
# device.py
# Python bindings for libparted (built on top of the _ped Python module).
#
# Copyright The pyparted Project Authors
# SPDX-License-Identifier: GPL-2.0-or-later
#

import sys

if sys.version_info >= (3,):
    long = int

import math
from decimal import Decimal
import warnings

import parted
import _ped

from parted.decorators import localeC


class Device(object):
    """Device()

    Device represents a physical piece of hardware in the system, e.g. a
    disk.  A Device should be considered a low-level and operating system
    specific interface to the hardware.

    A number of read-only properties of the Device are available.

    For information on the individual methods, see help(Device.METHODNAME)"""

    @localeC
    def __init__(self, path=None, PedDevice=None):
        """Create a new Device object based on the specified path or the
        already existing _ped.Device object.  You must provide either a
        path (e.g., "/dev/sda") or an existing _ped.Device object, but
        not both."""

        if PedDevice:
            self.__device = PedDevice
        elif path is not None:
            self.__device = _ped.device_get(path)
        else:
            raise parted.DeviceException("no path or PedDevice specified")

    def __eq__(self, other):
        return not self.__ne__(other)

    def __ne__(self, other):
        if not isinstance(self, other.__class__):
            return True

        return (
            self.model != other.model
            or self.path != other.path
            or self.type != other.type
            or self.length != other.length
        )

    def __getCHS(self, geometry):
        return (geometry.cylinders, geometry.heads, geometry.sectors)

    @property
    def model(self):
        """Model name and vendor of this device."""
        return self.__device.model

    @property
    def path(self):
        """Filesystem node path of this device (e.g., /dev/sda)."""
        return self.__device.path

    @property
    def type(self):
        """Type of this device.  An integer constant corresponding
        to one of the parted.DEVICE_* values.
        """
        return self.__device.type

    @property
    def sectorSize(self):
        """Sector size (in bytes) for this device."""
        return self.__device.sector_size

    @property
    def physicalSectorSize(self):
        """Physical sector size (in bytes) for this device.  Not always
        the same as sectorSize, but is a multiple of sectorSize.
        """
        return self.__device.phys_sector_size

    @property
    def length(self):
        """The size of this device in sectors."""
        return self.__device.length

    @property
    def openCount(self):
        """How many times the open() method has been called on this device."""
        return self.__device.open_count

    @property
    def readOnly(self):
        """True if the device is currently in read-only mode, False
        otherwise.
        """
        return bool(self.__device.read_only)

    @property
    def externalMode(self):
        """True if external access mode is currently activated on this
        device, False otherwise.  External access mode has to be used
        if you want to use an external command on the device while
        you are currently using it in pyparted.
        """
        return bool(self.__device.external_mode)

    @property
    def dirty(self):
        """True if the device is marked dirty, False otherwise."""
        return bool(self.__device.dirty)

    @property
    def bootDirty(self):
        """True if the device is marked boot dirty, False otherwise."""
        return bool(self.__device.boot_dirty)

    @property
    def host(self):
        """The host value of this device."""
        return self.__device.host

    @property
    def did(self):
        """The did value of this device."""
        return self.__device.did

    @property
    def busy(self):
        """True if this device is busy, False otherwise."""
        return self.__device.is_busy()

    @property
    def hardwareGeometry(self):
        """A 3-tuple representing the hardware geometry of this device.
        The tuple is in order of cylinders, heads, and sectors.
        """
        return self.__getCHS(self.__device.hw_geom)

    @property
    def biosGeometry(self):
        """A 3-tuple representing the BIOS geometry of this device.
        The tuple is in order of cylinders, heads, and sectors.
        """
        return self.__getCHS(self.__device.bios_geom)

    def __str__(self):
        s = (
            "parted.Device instance --\n"
            "  model: %(model)s  path: %(path)s  type: %(type)s\n"
            "  sectorSize: %(sectorSize)s  physicalSectorSize:  %(physSectorSize)s\n"
            "  length: %(length)s  openCount: %(openCount)s  readOnly: %(readOnly)s\n"
            "  externalMode: %(external)s  dirty: %(dirty)s  bootDirty: %(bootDirty)s\n"
            "  host: %(host)s  did: %(did)s  busy: %(busy)s\n"
            "  hardwareGeometry: %(hardwareGeom)s  biosGeometry: %(biosGeom)s\n"
            "  PedDevice: %(ped)r"
            % {
                "model": self.model,
                "path": self.path,
                "type": self.type,
                "sectorSize": self.sectorSize,
                "physSectorSize": self.physicalSectorSize,
                "length": self.length,
                "openCount": self.openCount,
                "readOnly": self.readOnly,
                "external": self.externalMode,
                "dirty": self.dirty,
                "bootDirty": self.bootDirty,
                "host": self.host,
                "did": self.did,
                "busy": self.busy,
                "hardwareGeom": self.hardwareGeometry,
                "biosGeom": self.biosGeometry,
                "ped": self.__device,
            }
        )
        return s

    @localeC
    def clobber(self):
        """Remove all identifying signatures of the partition table."""
        return self.__device.clobber()

    @localeC
    def open(self):
        """Open this Device for read operations."""

        return self.__device.open()

    @localeC
    def close(self):
        """Close this Device.  Used after open() method calls."""

        return self.__device.close()

    @localeC
    def destroy(self):
        """Destroy this Device.  Operating system specific."""

        return self.__device.destroy()

    @localeC
    def removeFromCache(self):
        """Remove this Device from the internal libparted device cache."""

        return self.__device.cache_remove()

    @localeC
    def beginExternalAccess(self):
        """Set up the Device for use by an external program.  Call this method
        before running an external program that uses the Device."""

        return self.__device.begin_external_access()

    @localeC
    def endExternalAccess(self):
        """Turn off external access mode for the Device.  Call this method once
        your external program has finished using the Device."""

        return self.__device.end_external_access()

    @localeC
    def read(self, start, count):
        """From the sector indentified by start, read and return count sectors
        from the Device."""

        return self.__device.read(start, count)

    @localeC
    def write(self, buf, start, count):
        """From the sector identified by start, write count sectors from
        buffer to the Device."""

        return self.__device.write(buf, start, count)

    @localeC
    def sync(self, fast=False):
        """Perform a operating-system specific sync(2) operation on the
        Device.  If fast is True, try to perform a fast sync(2)."""

        if fast:
            return self.__device.sync_fast()
        else:
            return self.__device.sync()

    @localeC
    def check(self, start, count):
        """From the sector identified by start, perform an operating
        system specific check on count sectors."""
        return self.__device.check(start, count)

    @localeC
    def startSectorToCylinder(self, sector):
        """Return the closest cylinder (round down) to sector on
        this Device."""
        (_, heads, sectors) = self.biosGeometry
        return long(math.floor((Decimal(sector) / (heads * sectors)) + 1))

    @localeC
    def endSectorToCylinder(self, sector):
        """Return the closest cylinder (round up) to sector on
        this Device."""
        (_, heads, sectors) = self.biosGeometry
        return long(math.ceil(Decimal((sector + 1)) / (heads * sectors)))

    @localeC
    def startCylinderToSector(self, cylinder):
        """Return the sector corresponding to cylinder as a
        starting cylinder on this Device."""
        (_, heads, sectors) = self.biosGeometry
        return long((cylinder - 1) * (heads * sectors))

    @localeC
    def endCylinderToSector(self, cylinder):
        """Return the sector corresponding to cylinder as an
        ending cylinder on this Device."""
        (_, heads, sectors) = self.biosGeometry
        return long(((cylinder) * (heads * sectors)) - 1)

    def getSize(self, unit="MB"):
        """Return the size of the Device in the unit specified.  The unit
        is given as a string corresponding to one of the following
        abbreviations:  b (bytes), KB (kilobytes), MB (megabytes), GB
        (gigabytes), TB (terabytes).  An invalid unit string will raise a
        SyntaxError exception.  The default unit is MB."""
        warnings.warn("use the getLength method", DeprecationWarning)
        lunit = unit.lower()

        if lunit not in parted._exponent.keys():
            raise SyntaxError("invalid unit %s given" % (unit))

        size = Decimal(self.__device.length)
        size /= Decimal(math.pow(1024.0, parted._exponent[lunit]))
        size *= Decimal(self.sectorSize)

        return float(size)

    @localeC
    def getLength(self, unit="sectors"):
        """Return the length of the device in sectors. Optionally, a SI or
        IEC prefix followed by a 'B' may be given in order to convert the
        length into bytes. The allowed values include B, kB, MB, GB, TB, KiB,
        MiB, GiB, and TiB."""
        sectors = self.__device.length
        if unit == "sectors":
            return sectors
        return parted.formatBytes(sectors * self.sectorSize, unit)

    @localeC
    def getConstraint(self):
        """Return a Constraint defining the limitations imposed by
        this Device."""
        return parted.Constraint(PedConstraint=self.__device.get_constraint())

    @property
    @localeC
    def minimalAlignedConstraint(self):
        """Return a Constraint defining the limitations and minimal advisable
        alignment imposed by this Device."""
        constraint = self.__device.get_minimal_aligned_constraint()
        return parted.Constraint(PedConstraint=constraint)

    @property
    @localeC
    def optimalAlignedConstraint(self):
        """Return a Constraint defining the limitations and optimal
        alignment imposed by this Device."""
        constraint = self.__device.get_optimal_aligned_constraint()
        return parted.Constraint(PedConstraint=constraint)

    @property
    @localeC
    def minimumAlignment(self):
        """Return an Alignment defining the minimum alignment for this Device."""
        alignment = self.__device.get_minimum_alignment()
        return parted.Alignment(PedAlignment=alignment)

    @property
    @localeC
    def optimumAlignment(self):
        """Return an Alignment defining the optimum alignment for this Device."""
        alignment = self.__device.get_optimum_alignment()
        return parted.Alignment(PedAlignment=alignment)

    def getPedDevice(self):
        """Return the _ped.Device object contained in this Device.
        For internal module use only."""
        return self.__device



#
# geometry.py
# Python bindings for libparted (built on top of the _ped Python module).
#
# Copyright The pyparted Project Authors
# SPDX-License-Identifier: GPL-2.0-or-later
#

import math
import warnings

import parted
import _ped

from parted.decorators import localeC


class Geometry(object):
    """Geometry()

    Geometry represents a region on a device in the system - a disk or
    partition.  It is expressed in terms of a starting sector and a length.
    Many methods (read and write methods in particular) throughout pyparted
    take in a Geometry object as an argument."""

    @localeC
    def __init__(
        self, device=None, start=None, length=None, end=None, PedGeometry=None
    ):
        """Create a new Geometry object for the given _ped.Device that extends
        for length sectors from the start sector.  Optionally, an end sector
        can also be provided."""
        if PedGeometry:
            self.__geometry = PedGeometry

            if device is None:
                self._device = parted.Device(PedDevice=self.__geometry.dev)
            else:
                self._device = device
        elif not end:
            self._device = device
            self.__geometry = _ped.Geometry(self.device.getPedDevice(), start, length)
        elif not length and (end > start):
            self._device = device
            self.__geometry = _ped.Geometry(
                self.device.getPedDevice(), start, (end - start + 1), end=end
            )
        elif start and length and end and (end > start):
            self._device = device
            self.__geometry = _ped.Geometry(
                self.device.getPedDevice(), start, length, end=end
            )
        else:
            raise parted.GeometryException(
                "must specify PedGeometry or (device, start, length) or (device, start, end) or (device, start, length, end)"
            )

    def __eq__(self, other):
        return not self.__ne__(other)

    def __ne__(self, other):
        if not isinstance(self, other.__class__):
            return True

        return (
            self.device != other.device
            or self.start != other.start
            or self.length != other.length
        )

    def __str__(self):
        s = (
            "parted.Geometry instance --\n"
            "  start: %(start)s  end: %(end)s  length: %(length)s\n"
            "  device: %(device)r  PedGeometry: %(ped)r"
            % {
                "start": self.start,
                "end": self.end,
                "length": self.length,
                "device": self.device,
                "ped": self.__geometry,
            }
        )
        return s

    @property
    def device(self):
        """The Device this geometry describes."""
        return self._device

    start = property(
        lambda s: s.__geometry.start, lambda s, v: s.__geometry.set_start(v)
    )
    end = property(lambda s: s.__geometry.end, lambda s, v: s.__geometry.set_end(v))
    length = property(
        lambda s: s.__geometry.length,
        lambda s, v: s.__geometry.set(s.__geometry.start, v),
    )

    @localeC
    def check(self, offset, granularity, count, timer=None):
        """Check the region described by self for errors on the disk.
        offset -- The beginning of the region to check, in sectors from the
                  start of the geometry.
        granularity -- How sectors should be grouped together
        count -- How many sectors from the region to check."""
        if not timer:
            return self.__geometry.check(offset, granularity, count)
        else:
            return self.__geometry.check(offset, granularity, count, timer)

    @localeC
    def contains(self, b):
        """Return whether Geometry b is contained entirely within self and on
        the same physical device."""
        return self.__geometry.test_inside(b.getPedGeometry())

    @localeC
    def containsSector(self, sector):
        """Return whether the sectory is contained entirely within self."""
        return self.__geometry.test_sector_inside(sector)

    @localeC
    def getSize(self, unit="MB"):
        """Return the size of the geometry in the unit specified.  The unit
        is given as a string corresponding to one of the following
        abbreviations:  b (bytes), KB (kilobytes), MB (megabytes), GB
        (gigabytes), TB (terabytes).  An invalid unit string will raise a
        SyntaxError exception.  The default unit is MB."""
        warnings.warn("use the getLength method", DeprecationWarning)
        lunit = unit.lower()
        size = self.length * self.device.sectorSize

        if lunit not in parted._exponent.keys():
            raise SyntaxError("invalid unit %s given" % (unit))

        return size / math.pow(1024.0, parted._exponent[lunit])

    @localeC
    def getLength(self, unit="sectors"):
        """Return the length of the geometry in sectors. Optionally, a SI or
        IEC prefix followed by a 'B' may be given in order to convert the
        length into bytes. The allowed values include B, kB, MB, GB, TB, KiB,
        MiB, GiB, and TiB."""
        sectors = self.length
        if unit == "sectors":
            return sectors
        return parted.formatBytes(sectors * self.device.sectorSize, unit)

    @localeC
    def intersect(self, b):
        """Return a new Geometry describing the region common to both self
        and Geometry b.  Raises ArithmeticError if the regions do not
        intersect."""
        return Geometry(PedGeometry=self.__geometry.intersect(b.getPedGeometry()))

    @localeC
    def map(self, src, sector):
        """Given a Geometry src that overlaps with self and a sector inside src,
        this method translates the address of the sector into an address
        inside self.  If self does not contain sector, ArithmeticError will
        be raised."""
        return parted.Geometry(
            PedGeometry=self.__geometry.map(src.getPedGeometry(), sector)
        )

    @localeC
    def overlapsWith(self, b):
        """Return whether self and b are on the same device and share at least
        some of the same region."""
        try:
            self.__geometry.intersect(b.getPedGeometry())
            return True
        except ArithmeticError:
            return False

    @localeC
    def read(self, offset, count):
        """Read data from the region described by self.
        offset -- The number of sectors from the beginning of the region
                  (not the beginning of the disk) to read.
        count  -- The number of sectors to read."""
        return self.__geometry.read(offset, count)

    @localeC
    def sync(self, fast=False):
        """Flushes all caches on the device described by self.  If fast is
        True, the flush will be quicked by cache coherency is not
        guaranteed."""
        if fast:
            return self.__geometry.sync_fast()
        else:
            return self.__geometry.sync()

    @localeC
    def write(self, buf, offset, count):
        """Write data into the region described by self.
        buf    -- The data to be written.
        offset -- Where to start writing to region, expressed as the number
                  of sectors from the start of the region (not the disk).
        count  -- How many sectors of buf to write out."""
        return self.__geometry.write(buf, offset, count)

    def getPedGeometry(self):
        """Return the _ped.Geometry object contained in this Geometry.
        For internal module use only."""
        return self.__geometry
